#!/usr/bin/env python
#coding=utf-8

# Copyright (C) 2011, Alibaba Cloud Computing

#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import platform
import urllib
import base64
import hmac
from hashlib import sha1 as sha
import os
import StringIO
from threading import Thread
import Queue
import threading
import ConfigParser
import logging
from logging.handlers import RotatingFileHandler
from xml.sax.saxutils import escape
import socket
import sys
import calendar
import imp
try:
    from oss.oss_xml_handler import *
except:
    from oss_xml_handler import *

def get_md5():
    if sys.version_info >= (2, 6):
        import hashlib
        hash = hashlib.md5()
    else:
        import md5
        hash = md5.new()
    return hash

#LOG_LEVEL can be one of DEBUG INFO ERROR CRITICAL WARNNING
DEBUG = False 
LOG_LEVEL = "DEBUG" 
PROVIDER = "OSS"
SELF_DEFINE_HEADER_PREFIX = "x-oss-"
if "AWS" == PROVIDER:
    SELF_DEFINE_HEADER_PREFIX = "x-amz-"
OSS_HOST_LIST = ["aliyun-inc.com", "aliyuncs.com", "alibaba.net", "s3.amazonaws.com"]
class AtomicInt:
    """
    only the '+=' is atomic
    """
    def __init__(self, v = 0):
        self.lock = threading.Lock()
        self.value = v

    def __add__(self, v):
        return AtomicInt(self.value + v)

    def __radd__(self, v):
        return AtomicInt(self.value + v)

    def __iadd__(self, v):
        self.lock.acquire()
        self.value += v
        self.lock.release()
        return self

    def __repr__(self):
        return str(self.value)

    def __sub__(self, v):
        return AtomicInt(self.value - v)

    def __rsub__(self, v):
        return AtomicInt(v - self.value)

    def __cmp__(self, v):
        if self.value < v:
            return -1
        elif self.value == v:
            return 0
        else: 
            return 1

    def __mod__(self, v):
        return (self.value % v)

class EmptyHandler(logging.Handler):
    def __init__(self):
        self.lock = None
        self.level = None
    def emit(self, record):
        pass
    def handle(self, record):
        pass
    def createLock(self):
        self.lock = None 

class Logger():
    def __init__(self, debug, log_name, log_level, logger):
        self.logger = logging.getLogger(logger)
        if debug:
            logfile = os.path.join(os.getcwd(), log_name)
            max_log_size = 100*1024*1024 #Bytes
            backup_count = 5
            format = \
            "%(asctime)s %(levelname)-8s[%(filename)s:%(lineno)d(%(funcName)s)] %(message)s"
            hdlr = RotatingFileHandler(logfile,
                                          mode='a',
                                          maxBytes=max_log_size,
                                          backupCount=backup_count)
            formatter = logging.Formatter(format)
            hdlr.setFormatter(formatter)
            self.logger.addHandler(hdlr)
            if "DEBUG" == log_level.upper():
                self.logger.setLevel(logging.DEBUG)
            elif "INFO" == log_level.upper():
                self.logger.setLevel(logging.INFO)
            elif "WARNING" == log_level.upper():
                self.logger.setLevel(logging.WARNING)
            elif "ERROR" == log_level.upper():
                self.logger.setLevel(logging.ERROR)
            elif "CRITICAL" == log_level.upper():
                self.logger.setLevel(logging.CRITICAL)
            else:
                self.logger.setLevel(logging.ERROR)
        else:
            self.logger.addHandler(EmptyHandler())

    def getlogger(self):
        return self.logger

OSS_LOGGER_SET = None 
PART_UPLOAD_OK = AtomicInt()
PART_UPLOAD_FAIL = AtomicInt()

def convert_to_localtime(osstimestamp, format="%Y-%m-%dT%H:%M:%S.000Z"):
    ts = format_unixtime(osstimestamp, format)
    return time.strftime("%Y-%m-%d %X", time.localtime(ts))

def format_unixtime(osstimestamp, format="%Y-%m-%dT%H:%M:%S.000Z"):
    imp.acquire_lock()
    try:
        ts = (int)(calendar.timegm(time.strptime(osstimestamp, format)))
    except:
        print "format_unixtime:%s exception, %s, %s" % (osstimestamp, sys.exc_info()[0], sys.exc_info()[1])
    imp.release_lock()
    return ts

def helper_get_host_from_resp(res, bucket):
    host = helper_get_host_from_headers(res.getheaders(), bucket)
    if not host:
        xml = res.read()
        host = RedirectXml(xml).Endpoint().strip()
        host = helper_get_host_from_endpoint(host, bucket)
    return host

def helper_get_host_from_headers(headers, bucket):
    mp = convert_header2map(headers)
    location = safe_get_element('location', mp).strip()
    #https://bucket.oss.aliyuncs.com or http://oss.aliyuncs.com/bucket
    location = location.replace("https://", "").replace("http://", "")
    if location.startswith("%s." % bucket):
        location = location[len(bucket)+1:]
    index = location.find('/')
    if index == -1:
        return location
    return location[:index]

def helper_get_host_from_endpoint(host, bucket):
    index = host.find('/')
    if index != -1:
        host = host[:index]
    index = host.find('\\')
    if index != -1:
        host = host[:index]
    index = host.find(bucket)
    if index == 0:
        host = host[len(bucket)+1:]
    return host

def check_bucket_valid(bucket):
    alphabeta = "abcdefghijklmnopqrstuvwxyz0123456789-"
    if len(bucket) < 3 or len(bucket) > 63:
        return False
    if bucket[-1] == "-" or bucket[-1] == "_":
        return False
    if not ((bucket[0] >= 'a' and bucket[0] <= 'z') or (bucket[0] >= '0' and bucket[0] <= '9')):
        return False
    for i in bucket:
        if not i in alphabeta:
            return False
    return True

def check_redirect(res):
    is_redirect = False
    try:
        if res.status == 301 or res.status == 302:
            is_redirect = True
    except:
        pass
    return is_redirect

########## function for Authorization ##########
def _format_header(headers=None):
    '''
    format the headers that self define
    convert the self define headers to lower.
    '''
    if not headers:
        headers = {}
    tmp_headers = {}
    for k in headers.keys():
        if isinstance(headers[k], unicode):
            headers[k] = convert_utf8(headers[k])

        if k.lower().startswith(SELF_DEFINE_HEADER_PREFIX):
            k_lower = k.lower().strip()
            tmp_headers[k_lower] = headers[k]
        else:
            tmp_headers[k.strip()] = headers[k]
    return tmp_headers

def get_assign(secret_access_key, method, headers=None, resource="/", result=None, debug=DEBUG):
    '''
    Create the authorization for OSS based on header input.
    You should put it into "Authorization" parameter of header.
    '''
    if not headers:
        headers = {}
    if not result:
        result = []
    content_md5 = ""
    content_type = ""
    date = ""
    canonicalized_oss_headers = ""
    secret_access_key = convert_utf8(secret_access_key)
    global OSS_LOGGER_SET 
    if not OSS_LOGGER_SET:
        OSS_LOGGER_SET = Logger(debug, "log.txt", LOG_LEVEL, "oss_util").getlogger()
    OSS_LOGGER_SET.debug("secret_access_key: %s" % secret_access_key)
    content_md5 = safe_get_element('Content-MD5', headers)
    content_type = safe_get_element('Content-Type', headers)
    date = safe_get_element('Date', headers)
    canonicalized_resource = resource
    tmp_headers = _format_header(headers)
    if len(tmp_headers) > 0:
        x_header_list = tmp_headers.keys()
        x_header_list.sort()
        for k in x_header_list:
            if k.startswith(SELF_DEFINE_HEADER_PREFIX):
                canonicalized_oss_headers += "%s:%s\n" % (k, tmp_headers[k]) 
    string_to_sign = method + "\n" + content_md5.strip() + "\n" + content_type + "\n" + date + "\n" + canonicalized_oss_headers + canonicalized_resource
    result.append(string_to_sign)
    OSS_LOGGER_SET.debug("method:%s\n content_md5:%s\n content_type:%s\n data:%s\n canonicalized_oss_headers:%s\n canonicalized_resource:%s\n" % (method, content_md5, content_type, date, canonicalized_oss_headers, canonicalized_resource))
    OSS_LOGGER_SET.debug("string_to_sign:%s\n \nlength of string_to_sign:%d\n" % (string_to_sign, len(string_to_sign)))
    h = hmac.new(secret_access_key, string_to_sign, sha)
    sign_result = base64.encodestring(h.digest()).strip()
    OSS_LOGGER_SET.debug("sign result:%s" % sign_result)
    return sign_result

def get_resource(params=None):
    if not params:
        return ""
    tmp_headers = {}
    for k, v in params.items():
        tmp_k = k.lower().strip()
        tmp_headers[tmp_k] = v
    override_response_list = ['response-content-type', 'response-content-language', 
                              'response-cache-control', 'logging', 'response-content-encoding', 
                              'acl', 'uploadId', 'uploads', 'partNumber', 'group', 'link', 
                              'delete', 'website', 'location', 'objectInfo',
                              'response-expires', 'response-content-disposition', 'cors', 'lifecycle',
                              'restore', 'qos', 'referer']
    override_response_list.sort()
    resource = ""
    separator = "?"
    for i in override_response_list:
        if tmp_headers.has_key(i.lower()):
            resource += separator
            resource += i
            tmp_key = str(tmp_headers[i.lower()])
            if len(tmp_key) != 0:
                resource += "="
                resource += tmp_key 
            separator = '&'
    return resource

def oss_quote(in_str):
    if not isinstance(in_str, str):
        in_str = str(in_str)
    return urllib.quote(in_str, '')

def append_param(url, params):
    '''
    convert the parameters to query string of URI.
    '''
    l = []
    for k, v in params.items():
        k = k.replace('_', '-')
        if  k == 'maxkeys':
            k = 'max-keys'
        v = convert_utf8(v) 
        if v is not None and v != '':
            l.append('%s=%s' % (oss_quote(k), oss_quote(v)))
        elif k == 'acl':
            l.append('%s' % (oss_quote(k)))
        elif v is None or v == '':
            l.append('%s' % (oss_quote(k)))
    if len(l):
        url = url + '?' + '&'.join(l)
    return url

############### Construct XML ###############
def create_object_group_msg_xml(part_msg_list=None):
    '''
    get information from part_msg_list and covert it to xml.
    part_msg_list has special format.
    '''
    if not part_msg_list:
        part_msg_list = []
    xml_string = r'<CreateFileGroup>'
    for part in part_msg_list:
        if len(part) >= 3:
            if isinstance(part[1], unicode):
                file_path = convert_utf8(part[1])
            else:
                file_path = part[1]
            file_path = escape(file_path)
            xml_string += r'<Part>'
            xml_string += r'<PartNumber>' + str(part[0]) + r'</PartNumber>'
            xml_string += r'<PartName>' + str(file_path) + r'</PartName>'
            xml_string += r'<ETag>"' + str(part[2]).upper() + r'"</ETag>'
            xml_string += r'</Part>'
        else:
            print "the ", part, " in part_msg_list is not as expected!"
            return ""
    xml_string += r'</CreateFileGroup>'

    return xml_string

def create_object_link_msg_xml_by_name(object_list = None):
    '''
    get information from object_list and covert it to xml.
    '''
    if not object_list:
        object_list = []
    xml_string = r'<CreateObjectLink>'
    for i in range(len(object_list)):
        part = str(object_list[i]).strip()
        file_path = convert_utf8(part)
        file_path = escape(file_path)
        xml_string += r'<Part>'
        xml_string += r'<PartNumber>' + str(i + 1) + r'</PartNumber>'
        xml_string += r'<PartName>' + str(file_path) + r'</PartName>'
        xml_string += r'</Part>'
    xml_string += r'</CreateObjectLink>'

    return xml_string

def create_object_link_msg_xml(part_msg_list = None):
    '''
    get information from part_msg_list and covert it to xml.
    part_msg_list has special format.
    '''
    if not part_msg_list:
        part_msg_list = []
    xml_string = r'<CreateObjectLink>'
    for part in part_msg_list:
        if len(part) >= 2:
            file_path = convert_utf8(part[1])
            file_path = escape(file_path)
            xml_string += r'<Part>'
            xml_string += r'<PartNumber>' + str(part[0]) + r'</PartNumber>'
            xml_string += r'<PartName>' + str(file_path) + r'</PartName>'
            xml_string += r'</Part>'
        else:
            print "the ", part, " in part_msg_list is not as expected!"
            return ""
    xml_string += r'</CreateObjectLink>'

    return xml_string

def create_part_xml(part_msg_list=None):
    '''
    get information from part_msg_list and covert it to xml.
    part_msg_list has special format.
    '''
    if not part_msg_list:
        part_msg_list = []
    xml_string = r'<CompleteMultipartUpload>'
    for part in part_msg_list:
        if len(part) >= 3:
            xml_string += r'<Part>'
            xml_string += r'<PartNumber>' + str(part[0]) + r'</PartNumber>'
            xml_string += r'<ETag>"' + str(part[2]).upper() + r'"</ETag>'
            xml_string += r'</Part>'
        else:
            print "the ", part, " in part_msg_list is not as expected!"
            return ""
    xml_string += r'</CompleteMultipartUpload>'

    return xml_string

def create_delete_object_msg_xml(object_list=None, is_quiet=False, is_defult=False):
    '''
    covert object name list to xml.
    '''
    if not object_list:
        object_list = []
    xml_string = r'<Delete>'
    if not is_defult:
        if is_quiet:
            xml_string += r'<Quiet>true</Quiet>'
        else:
            xml_string += r'<Quiet>false</Quiet>'
    for object in object_list:
        key = convert_utf8(object)
        key = escape(key)
        xml_string += r'<Object><Key>%s</Key></Object>' % key
    xml_string += r'</Delete>'
    return xml_string

############### operate OSS ###############
def clear_all_object_of_bucket(oss_instance, bucket):
    '''
    clean all objects in bucket, after that, it will delete this bucket.
    '''
    return clear_all_objects_in_bucket(oss_instance, bucket)

def clear_all_objects_in_bucket(oss_instance, bucket, delete_marker="", delete_upload_id_marker="", debug=False):
    '''
    it will clean all objects in bucket, after that, it will delete this bucket.

    example:
    from oss_api import *
    host = ""
    id = ""
    key = ""
    oss_instance = OssAPI(host, id, key)
    bucket = "leopublicreadprivatewrite"
    if clear_all_objects_in_bucket(oss_instance, bucket):
        pass
    else:
        print "clean Fail"
    '''
    prefix = ""
    delimiter = ""
    maxkeys = 1000
    try:
        delete_all_objects(oss_instance, bucket, prefix, delimiter, delete_marker, maxkeys, debug)
        delete_all_parts(oss_instance, bucket, delete_marker, delete_upload_id_marker, debug)
        res = oss_instance.delete_bucket(bucket)
        if (res.status / 100 != 2 and res.status != 404):
            print "clear_all_objects_in_bucket: delete bucket:%s fail, ret:%s, request id:%s" % (bucket, res.status, res.getheader("x-oss-request-id"))
            return False
    except socket.error:
        print "socket exception when clear_all_objects_in_bucket:%s from %s" % (bucket, oss_instance.host)
        return False
    return True

def delete_all_objects(oss_instance, bucket, prefix="", delimiter="", delete_marker="", maxkeys=1000, debug=False):
    marker = delete_marker
    delete_obj_num = 0
    while 1:
        object_list = []
        res = oss_instance.get_bucket(bucket, prefix, marker, delimiter, maxkeys)
        if res.status != 200:
            print 'list object in bucket fail'
            print res.status
            print res.read()
            return False
        body = res.read()
        (tmp_object_list, marker) = get_object_list_marker_from_xml(body)
        for item in tmp_object_list:
            object_list.append(item[0])

        if object_list:
            object_list_xml = create_delete_object_msg_xml(object_list)
            res = oss_instance.batch_delete_object(bucket, object_list_xml)
            if res.status/100 != 2:
                if marker:
                    print "delete_all_objects: batch delete objects in bucket:%s fail, ret:%s, request id:%s, first object:%s, marker:%s" % (bucket, res.status, res.getheader("x-oss-request-id"), object_list[0], marker)
                else:
                    print "delete_all_objects: batch delete objects in bucket:%s fail, ret:%s, request id:%s, first object:%s" % (bucket, res.status, res.getheader("x-oss-request-id"), object_list[0])
                return False
            else:
                if debug:
                    delete_obj_num += len(object_list)
                    if marker:
                        print "delete_all_objects: Now %s objects deleted, marker:%s" % (delete_obj_num, marker)
                    else:
                        print "delete_all_objects: Now %s objects deleted" % (delete_obj_num)
        if len(marker) == 0:
            break
    return True

def delete_all_parts(oss_instance, bucket, delete_object_marker="", delete_upload_id_marker="", debug=False):
    delete_mulitipart_num = 0
    marker = delete_object_marker
    id_marker = delete_upload_id_marker
    while 1:
        res = oss_instance.get_all_multipart_uploads(bucket, key_marker=marker, upload_id_marker=id_marker)
        if res.status != 200:
            break
        body = res.read()
        hh = GetMultipartUploadsXml(body)
        (fl, pl) = hh.list()
        for i in fl:
            object = convert_utf8(i[0])
            res = oss_instance.cancel_upload(bucket, object, i[1])
            if (res.status / 100 != 2 and res.status != 404):
                print "delete_all_parts: cancel upload object:%s, upload_id:%s FAIL, ret:%s, request-id:%s" % (object, i[1], res.status, res.getheader("x-oss-request-id"))
            else:
                delete_mulitipart_num += 1
                if debug:
                    print "delete_all_parts: cancel upload object:%s, upload_id:%s OK\nNow %s parts deleted." % (object, i[1], delete_mulitipart_num)
        if hh.is_truncated:
            marker = hh.next_key_marker
            id_marker = hh.next_upload_id_marker
        else:
            break
        if not marker:
            break

def clean_all_bucket(oss_instance):
    '''
    it will clean all bucket, including the all objects in bucket.
    '''
    res = oss_instance.get_service()
    ret = True
    if (res.status / 100) == 2:
        h = GetServiceXml(res.read())
        for b in h.bucket_list:
            if not clear_all_objects_in_bucket(oss_instance, b.name):
                print "clean bucket ", b.name, " failed! in clean_all_bucket"
                ret = False
        return ret
    else:
        print "failed! get service in clean_all_bucket return ", res.status
        print res.read()
        print res.getheaders()
        return False

def pgfs_clear_all_objects_in_bucket(oss_instance, bucket):
    '''
    it will clean all objects in bucket, after that, it will delete this bucket.
    '''
    b = GetAllObjects()
    b.get_all_object_in_bucket(oss_instance, bucket)
    for i in b.object_list:
        res = oss_instance.delete_object(bucket, i)
        if (res.status / 100 != 2):
            print "clear_all_objects_in_bucket: delete object fail, ret is:", res.status, "bucket is:", bucket, "object is: ", i
            return False
        else:
            pass
    res = oss_instance.delete_bucket(bucket)
    if (res.status / 100 != 2 and res.status != 404):
        print "clear_all_objects_in_bucket: delete bucket fail, ret is: %s, request id is:%s" % (res.status, res.getheader("x-oss-request-id"))
        return False
    return True

def pgfs_clean_all_bucket(oss_instance):
    '''
    it will clean all bucket, including the all objects in bucket.
    '''
    res = oss_instance.get_service()
    if (res.status / 100) == 2:
        h = GetServiceXml(res.read())
        for b in h.bucket_list:
            if not pgfs_clear_all_objects_in_bucket(oss_instance, b.name):
                print "clean bucket ", b.name, " failed! in clean_all_bucket"
                return False
        return True
    else:
        print "failed! get service in clean_all_bucket return ", res.status
        print res.read()
        print res.getheaders()
        return False

def delete_all_parts_of_object_group(oss, bucket, object_group_name):
    res = oss.get_object_group_index(bucket, object_group_name)
    if res.status == 200:
        body = res.read()
        h = GetObjectGroupIndexXml(body)
        object_group_index = h.list()
        for i in object_group_index:
            if len(i) == 4 and len(i[1]) > 0:
                part_name = i[1].strip()
                res = oss.delete_object(bucket, part_name)
                if res.status != 204:
                    print "delete part ", part_name, " in bucket:", bucket, " failed!"
                    return False
    else:
        return False
    return True

def delete_all_parts_of_object_link(oss, bucket, object_link_name):
    res = oss.get_link_index(bucket, object_link_name)
    if res.status == 200:
        body = res.read()
        h = GetObjectLinkIndexXml(body)
        object_link_index = h.list()
        for i in object_link_index:
            if len(i) == 2 and len(i[1]) > 0:
                part_name = i[1].strip()
                res = oss.delete_object(bucket, part_name)
                if res.status != 204:
                    print "delete part ", part_name, " in bucket:", bucket, " failed!"
                    return False
    else:
        return False
    return True

class GetAllObjects:
    def __init__(self):
        self.object_list = []
        self.dir_list = []

    def get_object_in_bucket(self, oss, bucket="", marker="", prefix=""):
        object_list = []
        maxkeys = 1000
        try:
            res = oss.get_bucket(bucket, prefix, marker, maxkeys=maxkeys)
            body = res.read()
            hh = GetBucketXml(body)
            (fl, pl) = hh.list()
            if len(fl) != 0:
                for i in fl:
                    object = convert_utf8(i[0])
                    object_list.append(object)
            marker = hh.nextmarker
        except:
            pass
        return (object_list, marker)
    
    def get_object_dir_in_bucket(self, oss, bucket="", marker="", prefix="", delimiter=""):
        object_list = []
        dir_list = []
        maxkeys = 1000
        try:
            res = oss.get_bucket(bucket, prefix, marker, delimiter, maxkeys=maxkeys)
            body = res.read()
            hh = GetBucketXml(body)
            (fl, pl) = hh.list()
            if len(fl) != 0:
                for i in fl:
                    object_list.append((i[0], i[3], i[1]))  #name, size, modified_time
            if len(pl) != 0:
                for i in pl:
                    dir_list.append(i)
            marker = hh.nextmarker
        except:
            pass
        return (object_list, dir_list, marker)

    def get_all_object_in_bucket(self, oss, bucket="", marker="", prefix=""):
        marker2 = ""
        while True:
            (object_list, marker) = self.get_object_in_bucket(oss, bucket, marker2, prefix)
            marker2 = marker
            if len(object_list) != 0:
                self.object_list.extend(object_list)
            if not marker:
                break

    def get_all_object_dir_in_bucket(self, oss, bucket="", marker="", prefix="", delimiter=""):
        marker2 = ""
        while True:
            (object_list, dir_list, marker) = self.get_object_dir_in_bucket(oss, bucket, marker2, prefix, delimiter)
            marker2 = marker
            if len(object_list) != 0:
                self.object_list.extend(object_list)
            if len(dir_list) != 0:
                self.dir_list.extend(dir_list)
            if not marker:
                break
        return (self.object_list, self.dir_list)

def get_all_buckets(oss):
    bucket_list = []
    res = oss.get_service()
    if res.status == 200:
        h = GetServiceXml(res.read())
        for b in h.bucket_list:
            bucket_list.append(str(b.name).strip())
    return bucket_list 

def get_object_list_marker_from_xml(body):
    #return ([(object_name, object_length, last_modify_time)...], marker)
    object_meta_list = []
    next_marker = ""
    hh = GetBucketXml(body)
    (fl, pl) = hh.list()
    if len(fl) != 0:
        for i in fl:
            object = convert_utf8(i[0])
            last_modify_time = i[1]
            length = i[3]
            etag = i[2]
            object_meta_list.append((object, length, last_modify_time, etag))
    if hh.is_truncated:
        next_marker = hh.nextmarker
    return (object_meta_list, next_marker)

def get_dir_list_marker_from_xml(body):
    #return (dirname, marker)
    dir_list = []
    next_marker = ""
    hh = GetBucketXml(body)
    (fl, pl) = hh.list()
    if len(pl) != 0:
        for i in pl:
            dir_list.append(i)
    if hh.is_truncated:
        next_marker = hh.nextmarker
    return (dir_list, next_marker)

def get_bucket_meta_list_marker_from_xml(body):
    next_marker = ""
    hh = GetServiceXml(body)
    if hh.is_truncated:
        next_marker = hh.nextmarker
    return (hh.bucket_list, next_marker)

def get_upload_id(oss, bucket, object, headers=None):
    '''
    get the upload id of object.
    Returns:
            string
    '''
    if not headers:
        headers = {}
    upload_id = ""
    res = oss.init_multi_upload(bucket, object, headers)
    if res.status == 200:
        body = res.read()
        h = GetInitUploadIdXml(body)
        upload_id = h.upload_id
    else:
        print res.status
        print res.getheaders()
        print res.read()
    return upload_id

def get_all_upload_id_list(oss, bucket, prefix=None):
    '''
    get all upload id of bucket
    Returns:
            list
    '''
    all_upload_id_list = []
    marker = ""
    id_marker = ""
    while True:
        res = oss.get_all_multipart_uploads(bucket, key_marker=marker, prefix=prefix, upload_id_marker=id_marker)
        if res.status != 200:
            return all_upload_id_list

        body = res.read()
        hh = GetMultipartUploadsXml(body)
        (fl, pl) = hh.list()
        for i in fl:
            all_upload_id_list.append(i)
        if hh.is_truncated:
            marker = hh.next_key_marker
            id_marker = hh.next_upload_id_marker
        else:
            break
        if not marker and not id_marker:
            break
    return all_upload_id_list

def get_upload_id_list(oss, bucket, object):
    '''
    get all upload id list of one object.
    Returns:
            list
    '''
    all_upload_id_list = []
    all_upload_id_list = get_all_upload_id_list(oss, bucket, object)
    upload_id_list = []
    if all_upload_id_list:
        for i in all_upload_id_list:
            if "%s" % convert_utf8(object) == "%s" % convert_utf8(i[0]):
                upload_id_list.append(i[1])
    return upload_id_list

def get_part_list(oss, bucket, object, upload_id, max_part=""):
    '''
    get uploaded part list of object.
    Returns:
            list
    '''
    part_list = []
    marker = ""
    while True:
        res = oss.get_all_parts(bucket, object, upload_id, part_number_marker = marker, max_parts=max_part)
        if res.status != 200:
            break
        body = res.read()
        h = GetPartsXml(body)
        part_list.extend(h.list())
        if h.is_truncated:
            marker = h.next_part_number_marker
        else:
            break
        if not marker:
            break
    return part_list

def get_part_xml(oss, bucket, object, upload_id):
    '''
    get uploaded part list of object.
    Returns:
            string
    '''
    part_list = []
    part_list = get_part_list(oss, bucket, object, upload_id)
    xml_string = r'<CompleteMultipartUpload>'
    for part in part_list:
        xml_string += r'<Part>'
        xml_string += r'<PartNumber>' + str(part[0]) + r'</PartNumber>'
        xml_string += r'<ETag>' + part[1] + r'</ETag>'
        xml_string += r'</Part>'
    xml_string += r'</CompleteMultipartUpload>'
    return xml_string

def get_part_map(oss, bucket, object, upload_id):
    part_list = []
    part_list = get_part_list(oss, bucket, object, upload_id)
    part_map = {}
    for part in part_list:
        part_number = str(part[0])
        etag = part[1]
        part_map[part_number] = etag
    return part_map

########## multi-thread ##########
def multi_get(oss, bucket, object, localfile, thread_num, retry_times):
    length = 0
    oss_md5string = ''
    res = oss.head_object(bucket, object)
    if 200 == res.status:
        length = (int)(res.getheader('content-length'))
        oss_md5string = res.getheader('x-oss-meta-md5')
    else:
        print "can not get the length of object:", object
        return False
    ranges = []
    ranges.append(0)
    size = length // thread_num
    for i in xrange(thread_num - 1):
        ranges.append((i + 1) * size)
    ranges.append(length)

    threadpool = []
    for i in xrange(len(ranges) - 1):
        exec("file_%s = open(localfile, 'wb+')" % i)
        exec("current = MultiGetWorker(oss, bucket, object, file_%s, ranges[i], ranges[i + 1] - 1, %s)" % (i, retry_times))
        threadpool.append(current)
        current.start()

    for item in threadpool:
        item.join()
    if oss_md5string != None:
        local_md5string, base64md5 = get_file_md5(localfile)
        if local_md5string != oss_md5string:
            print "localfile:%s md5:%s is not equal with object:%s md5:%s " % (localfile, local_md5string, object, oss_md5string)
            return False
    if not os.path.isfile(localfile) or length != os.path.getsize(localfile):
        print "localfile:%s size:%s is not equal with object:%s size:%s " % (localfile, os.path.getsize(localfile), object, length)
        return False
    else:
        return True

class DeleteObjectWorker(Thread):
    def __init__(self, oss, bucket, part_msg_list, retry_times=5):
        Thread.__init__(self)
        self.oss = oss
        self.bucket = bucket
        self.part_msg_list = part_msg_list
        self.retry_times = retry_times

    def run(self):
        bucket = self.bucket
        object_list = self.part_msg_list
        step = 1000
        begin = 0
        end = 0
        total_length = len(object_list)
        remain_length = total_length
        while True:
            if remain_length > step:
                end = begin + step
            elif remain_length > 0:
                end = begin + remain_length
            else:
                break
            is_fail = True
            retry_times = self.retry_times
            while True:
                try:
                    if retry_times <= 0:
                        break
                    res = self.oss.delete_objects(bucket, object_list[begin:end])
                    if res.status / 100 == 2:
                        is_fail = False
                        break
                except:
                    retry_times = retry_times - 1
                    time.sleep(1)
            if is_fail:
                print "delete object_list[%s:%s] failed!, first is %s" % (begin, end, object_list[begin])
            begin = end
            remain_length = remain_length - step

class PutObjectGroupWorker(Thread):
    def __init__(self, oss, bucket, file_path, part_msg_list, retry_times=5):
        Thread.__init__(self)
        self.oss = oss
        self.bucket = bucket
        self.part_msg_list = part_msg_list
        self.file_path = file_path
        self.retry_times = retry_times

    def run(self):
        for part in self.part_msg_list:
            if len(part) >= 5:
                bucket = self.bucket
                file_name = convert_utf8(part[1])
                object_name = file_name
                retry_times = self.retry_times
                is_skip = False
                while True:
                    try:
                        if retry_times <= 0:
                            break
                        res = self.oss.head_object(bucket, object_name)
                        if res.status == 200:
                            header_map = convert_header2map(res.getheaders())
                            etag = safe_get_element("etag", header_map)
                            md5_str = part[2]
                            if etag.replace('"', "").upper() == md5_str.upper():
                                is_skip = True
                        break
                    except:
                        retry_times = retry_times - 1
                        time.sleep(1)

                if is_skip:
                    continue

                partsize = part[3]
                offset = part[4]
                retry_times = self.retry_times
                while True:
                    try:
                        if retry_times <= 0:
                            break
                        res = self.oss.put_object_from_file_given_pos(bucket, object_name, self.file_path, offset, partsize)
                        if res.status != 200:
                            print "upload ", file_name, "failed!", " ret is:", res.status
                            print "headers", res.getheaders()
                            retry_times = retry_times - 1
                            time.sleep(1)
                        else:
                            break
                    except:
                        retry_times = retry_times - 1
                        time.sleep(1)

            else:
                print "ERROR! part", part , " is not as expected!"

class PutObjectLinkWorker(Thread):
    def __init__(self, oss, bucket, file_path, part_msg_list, retry_times=5):
        Thread.__init__(self)
        self.oss = oss
        self.bucket = bucket
        self.part_msg_list = part_msg_list
        self.file_path = file_path
        self.retry_times = retry_times

    def run(self):
        for part in self.part_msg_list:
            if len(part) >= 5:
                bucket = self.bucket
                file_name = convert_utf8(part[1])
                object_name = file_name
                retry_times = self.retry_times
                is_skip = False
                while True:
                    try:
                        if retry_times <= 0:
                            break
                        res = self.oss.head_object(bucket, object_name)
                        if res.status == 200:
                            header_map = convert_header2map(res.getheaders())
                            etag = safe_get_element("etag", header_map)
                            md5_str = part[2]
                            if etag.replace('"', "").upper() == md5_str.upper():
                                is_skip = True
                        break
                    except:
                        retry_times = retry_times - 1
                        time.sleep(1)

                if is_skip:
                    continue

                partsize = part[3]
                offset = part[4]
                retry_times = self.retry_times
                while True:
                    try:
                        if retry_times <= 0:
                            break
                        res = self.oss.put_object_from_file_given_pos(bucket, object_name, self.file_path, offset, partsize)
                        if res.status != 200:
                            print "upload ", file_name, "failed!", " ret is:", res.status
                            print "headers", res.getheaders()
                            retry_times = retry_times - 1
                            time.sleep(1)
                        else:
                            break
                    except:
                        retry_times = retry_times - 1
                        time.sleep(1)

            else:
                print "ERROR! part", part , " is not as expected!"

def multi_upload_file2(oss, bucket, object, filename, upload_id, thread_num=10, max_part_num=10000, retry_times=5, headers=None, params=None, debug=False, is_check_md5=False):
    if not upload_id:
        print "empty upload_id"
        return False
    part_msg_list = []
    if debug:
        print "split %s to get part list, it may take long time, please wait." % filename
    part_msg_list = split_large_file(filename, object, max_part_num, check_md5=is_check_md5)
    if debug:
        print "split %s finish." % filename
    queue = Queue.Queue(0)
    uploaded_part_map = {}
    part_msg_xml = create_part_xml(part_msg_list)
    each_part_retry_times = 1
    total_parts_num = len(part_msg_list)
    need_upload_parts_num = 0
    for i in range(retry_times):
        tmp_uploaded_part_map = get_part_map(oss, bucket, object, upload_id)
        if tmp_uploaded_part_map:
            for k, v in tmp_uploaded_part_map.items():
                uploaded_part_map[k] = v
        thread_pool = []
        for part in part_msg_list:
            if len(part) >= 5:
                part_number = str(part[0])
                md5_str = part[2]
                is_need_upload = True
                if uploaded_part_map.has_key(part_number):
                    md5_str = part[2]
                    if uploaded_part_map[part_number].replace('"', "").upper() == md5_str.upper():
                        is_need_upload = False
                        continue
                if is_need_upload:
                    queue.put((upload_part, oss, bucket, object, upload_id, filename, part, is_check_md5))
            else:
                print "not expected part", part
        need_upload_parts_num = queue.qsize()
        if debug:
            print "RetryTimes:%s, TotalParts:%s, NeedUploadParts:%s" % (i, total_parts_num, need_upload_parts_num)
        global PART_UPLOAD_OK
        global PART_UPLOAD_FAIL
        PART_UPLOAD_OK = AtomicInt()
        PART_UPLOAD_FAIL = AtomicInt()
                
        for i in xrange(thread_num):
            current = UploadPartWorker2(each_part_retry_times, queue, need_upload_parts_num, debug)
            thread_pool.append(current)
            current.start()
        queue.join()
        for item in thread_pool:
            item.join()

        res = oss.complete_upload(bucket, object, upload_id, part_msg_xml, headers, params)
        if res.status == 200:
            return res
        if res.status > 300 and res.status < 500:
            print res.read()
            raise Exception("-3, bad request, multi upload file failed! upload_id:%s" % (upload_id))
    raise Exception("-3, after retry %s, failed, multi upload file failed! upload_id:%s" % (retry_times, upload_id))
    
def upload_part(oss, bucket, object, upload_id, file_path, part, retry_times=2, is_check_md5=False):
    if len(part) == 6:
        part_number = str(part[0])
        md5_str = part[2]
        partsize = part[3]
        offset = part[4]
        base64md5_str = part[5]
        headers = {}
        if is_check_md5:
            headers["Content-MD5"] = base64md5_str
        for i in range(retry_times):
            try:
                res = oss.upload_part_from_file_given_pos(bucket, object, file_path, offset, partsize, upload_id, part_number, headers)
                if res.status != 200:
                    time.sleep(1)
                else:
                    return True
            except:
                time.sleep(1)
    else:
        print "not expected part for multiupload", part
    return False

class UploadPartWorker2(threading.Thread):
    def __init__(self, retry_times, queue, total_parts_num, debug):
        threading.Thread.__init__(self)
        self.queue = queue
        self.retry_times = retry_times
        self.debug = debug
        self.total_parts_num = total_parts_num

    def run(self):
        global PART_UPLOAD_OK
        global PART_UPLOAD_FAIL
        while 1:
            try:
                (upload_part, oss, bucket, object, upload_id, filename, part, is_check_md5) = self.queue.get(block=False)
                ret = upload_part(oss, bucket, object, upload_id, filename, part, self.retry_times, is_check_md5)
                if ret:
                    PART_UPLOAD_OK += 1
                else:
                    PART_UPLOAD_FAIL += 1
                sum = PART_UPLOAD_OK + PART_UPLOAD_FAIL

                if self.total_parts_num > 0:
                    exec("rate = 100*%s/(%s*1.0)" % (sum, self.total_parts_num))
                else:
                    rate = 0
                if self.debug:
                    print '\rOK:%s, FAIL:%s, TOTAL_DONE:%s, TOTAL_TO_DO:%s, PROCESS:%.2f%%' % (PART_UPLOAD_OK, PART_UPLOAD_FAIL, sum, self.total_parts_num, rate),
                    sys.stdout.flush()
                self.queue.task_done()
            except Queue.Empty:
                break
            except:
                PART_UPLOAD_FAIL += 1
                print sys.exc_info()[0], sys.exc_info()[1]
                self.queue.task_done()

class UploadPartWorker(Thread):
    def __init__(self, oss, bucket, object, upload_id, file_path, part_msg_list, uploaded_part_map, retry_times=5, debug=DEBUG):
        Thread.__init__(self)
        self.oss = oss
        self.bucket = bucket
        self.object = object
        self.part_msg_list = part_msg_list
        self.file_path = file_path
        self.upload_id = upload_id
        self.uploaded_part_map = uploaded_part_map.copy()
        self.retry_times = retry_times

    def run(self):
        for part in self.part_msg_list:
            part_number = str(part[0])
            if len(part) >= 5:
                bucket = self.bucket
                object = self.object
                partsize = part[3]
                offset = part[4]
                retry_times = self.retry_times
                while True:
                    try:
                        if self.uploaded_part_map.has_key(part_number):
                            md5_str = part[2]
                            if self.uploaded_part_map[part_number].replace('"', "").upper() == md5_str.upper():
                                break
                        if retry_times <= 0:
                            break
                        res = self.oss.upload_part_from_file_given_pos(bucket, object, self.file_path, offset, partsize, self.upload_id, part_number)
                        if res.status != 200:
                            retry_times = retry_times - 1
                            time.sleep(1)
                        else:
                            etag = res.getheader("etag")
                            if etag:
                                self.uploaded_part_map[part_number] = etag
                            break
                    except:
                        retry_times = retry_times - 1
                        time.sleep(1)
            else:
                print "not expected part for multiupload", part

class MultiGetWorker(Thread):
    def __init__(self, oss, bucket, object, file, start, end, retry_times=5):
        Thread.__init__(self)
        self.oss = oss
        self.bucket = bucket
        self.object = object
        self.curpos = start
        self.startpos = start
        self.endpos = end
        self.file = file
        self.length = self.endpos - self.startpos + 1
        self.get_buffer_size = 10*1024*1024
        self.retry_times = retry_times

    def run(self):
        if self.startpos > self.endpos:
            return
        retry_times = 0
        totalread = 0
        while True:
            headers = {}
            range_info = 'bytes=%d-%d' % (self.curpos, self.endpos)
            headers['Range'] = range_info
            self.file.seek(self.curpos)
            try:
                res = self.oss.object_operation("GET", self.bucket, self.object, headers)
                if res.status == 206:
                    while True:
                        content = res.read(self.get_buffer_size)
                        if content:
                            self.file.write(content)
                            totalread += len(content)
                            self.curpos += len(content)
                        else:
                            break
                else:
                    print "range get /%s/%s [%s] ret:%s" % (self.bucket, self.object, range_info, res.status)
            except:
                self.file.flush()
                print "range get /%s/%s [%s] exception, retry:%s" % (self.bucket, self.object, range_info, retry_times)
            if totalread == self.length or self.curpos > self.endpos:
                break
            retry_times += 1
            if retry_times > self.retry_times:
                print "ERROR, reach max retry times:%s when multi get /%s/%s" % (self.retry_times, self.bucket, self.object)
                break 
        self.file.flush()
        self.file.close()

############### misc ###############

def split_large_file(file_path, object_prefix="", max_part_num=1000, part_size=10*1024*1024, buffer_size=10*1024*1024, check_md5=False):
    parts_list = []

    if os.path.isfile(file_path):
        file_size = os.path.getsize(file_path)

        if file_size > part_size * max_part_num:
            part_size = (file_size + max_part_num - file_size % max_part_num) / max_part_num

        part_order = 1
        fp = open(file_path, 'rb')
        fp.seek(os.SEEK_SET)

        part_num = (file_size + part_size - 1) / part_size

        for i in xrange(0, part_num):
            left_len = part_size
            real_part_size = 0
            base64md5 = None
            m = get_md5()
            offset = part_size * i
            while True:
                read_size = 0
                if left_len <= 0:
                    break
                elif left_len < buffer_size:
                    read_size = left_len
                else:
                    read_size = buffer_size

                buffer_content = fp.read(read_size)
                m.update(buffer_content)
                real_part_size += len(buffer_content)

                left_len = left_len - read_size

            md5sum = m.hexdigest()
            if check_md5:
                base64md5 = base64.encodestring(m.digest()).strip()

            temp_file_name = os.path.basename(file_path) + "_" + str(part_order)
            object_prefix = convert_utf8(object_prefix)
            if not object_prefix:
                file_name = sum_string(temp_file_name) + "_" + temp_file_name
            else:
                file_name = object_prefix + "/" + sum_string(temp_file_name) + "_" + temp_file_name
            part_msg = (part_order, file_name, md5sum, real_part_size, offset, base64md5)
            parts_list.append(part_msg)
            part_order += 1

        fp.close()
    else:
        print "ERROR! No file: ", file_path, ", please check."

    return parts_list

BUFFER_SIZE = 10*1024*1024
def sumfile(fobj):
    '''Returns an md5 hash for an object with read() method.'''
    m = get_md5()
    while True:
        d = fobj.read(BUFFER_SIZE)
        if not d:
            break
        m.update(d)
    return m.hexdigest()

def md5sum(fname):
    '''Returns an md5 hash for file fname, or stdin if fname is "-".'''
    if fname == '-':
        ret = sumfile(sys.stdin)
    else:
        try:
            f = file(fname, 'rb')
            ret = sumfile(f)
            f.close()
        except:
            return 'Failed to get file:%s md5' % fname
    return ret

def md5sum2(filename, offset=0, partsize=0):
    m = get_md5()
    fp = open(filename, 'rb')
    if offset > os.path.getsize(filename):
        fp.seek(os.SEEK_SET, os.SEEK_END)
    else:
        fp.seek(offset)

    left_len = partsize
    BufferSize = BUFFER_SIZE
    while True:
        if left_len <= 0:
            break
        elif left_len < BufferSize:
            buffer_content = fp.read(left_len)
        else:
            buffer_content = fp.read(BufferSize)
        m.update(buffer_content)
        left_len = left_len - len(buffer_content)
    md5sum = m.hexdigest()
    return md5sum

def sum_string(content):
    f = StringIO.StringIO(content)
    md5sum = sumfile(f)
    f.close()
    return md5sum

def convert_header2map(header_list):
    header_map = {}
    for (a, b) in header_list:
        header_map[a] = b
    return header_map

def safe_get_element(name, container):
    for k, v in container.items():
        if k.strip().lower() == name.strip().lower():
            return v
    return ""

def get_content_type_by_filename(file_name):
    mime_type = ""
    mime_map = {}
    mime_map["js"] = "application/javascript"
    mime_map["xlsx"] = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
    mime_map["xltx"] = "application/vnd.openxmlformats-officedocument.spreadsheetml.template"
    mime_map["potx"] = "application/vnd.openxmlformats-officedocument.presentationml.template"
    mime_map["ppsx"] = "application/vnd.openxmlformats-officedocument.presentationml.slideshow"
    mime_map["pptx"] = "application/vnd.openxmlformats-officedocument.presentationml.presentation"
    mime_map["sldx"] = "application/vnd.openxmlformats-officedocument.presentationml.slide"
    mime_map["docx"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
    mime_map["dotx"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.template"
    mime_map["xlam"] = "application/vnd.ms-excel.addin.macroEnabled.12"
    mime_map["xlsb"] = "application/vnd.ms-excel.sheet.binary.macroEnabled.12"
    mime_map["apk"] = "application/vnd.android.package-archive"
    try:
        suffix = ""
        name = os.path.basename(file_name)
        suffix = name.split('.')[-1]
        if suffix in mime_map.keys():
            mime_type = mime_map[suffix] 
        else:
            import mimetypes
            mimetypes.init()
            mime_type = mimetypes.types_map["." + suffix]
    except Exception:
        mime_type = 'application/octet-stream'
    if not mime_type:
        mime_type = 'application/octet-stream'
    return mime_type

def smart_code(input_stream):
    if isinstance(input_stream, str):
        try:
            tmp = unicode(input_stream, 'utf-8')
        except UnicodeDecodeError:
            try:
                tmp = unicode(input_stream, 'gbk')
            except UnicodeDecodeError:
                try:
                    tmp = unicode(input_stream, 'big5')
                except UnicodeDecodeError:
                    try:
                        tmp = unicode(input_stream, 'ascii')
                    except:
                        tmp = input_stream
    else:
        tmp = input_stream
    return tmp

def is_ip(s):
    try:
        tmp_list = s.split(':')
        s = tmp_list[0]
        if s == 'localhost':
            return True
        tmp_list = s.split('.')
        if len(tmp_list) != 4:
            return False
        else:
            for i in tmp_list:
                if int(i) < 0 or int(i) > 255:
                    return False
    except:
        return False
    return True

def get_host_from_list(hosts):
    tmp_list = hosts.split(",")
    if len(tmp_list) <= 1:
        return hosts
    for tmp_host in tmp_list:
        tmp_host = tmp_host.strip()
        host = tmp_host
        port = 80
        try:
            host_port_list = tmp_host.split(":")
            if len(host_port_list) == 1:
                host = host_port_list[0].strip()
            elif len(host_port_list) == 2:
                host = host_port_list[0].strip()
                port = int(host_port_list[1].strip())
            sock=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
            sock.connect((host, port))
            return host
        except:
            pass
    return tmp_list[0].strip()
    
def is_oss_host(host, is_oss_host=False):
    if is_oss_host:
        return True
    for i in OSS_HOST_LIST: 
        if host.find(i) != -1:
            return True
    return False

def convert_utf8(input_string):
    if isinstance(input_string, unicode):
        input_string = input_string.encode('utf-8')
    return input_string

def get_string_base64_md5(string):
    fd = StringIO.StringIO(string)
    base64md5 = get_fp_base64_md5(fd)
    fd.close()
    return base64md5
    
def get_file_base64_md5(file):
    fd = open(file, 'rb')
    base64md5 = get_fp_base64_md5(fd)
    fd.close()
    return base64md5

def get_fp_base64_md5(fd):
    m = get_md5()
    while True:
        d = fd.read(BUFFER_SIZE)
        if not d:
            break
        m.update(d)
    base64md5 = base64.encodestring(m.digest()).strip()
    return base64md5

def get_file_md5(file):
    fd = open(file, 'rb')
    md5string, base64md5 = get_fp_md5(fd)
    fd.close()
    return md5string, base64md5

def get_fp_md5(fd):
    m = get_md5()
    while True:
        d = fd.read(BUFFER_SIZE)
        if not d:
            break
        m.update(d)
    md5string = m.hexdigest()
    base64md5 = base64.encodestring(m.digest()).strip()
    return md5string, base64md5

def get_unique_temp_filename(temp_dir, localfile):
    import random, string
    def random_str(randomlength=8):
        a = list(string.ascii_letters)
        random.shuffle(a)
        return ''.join(a[:randomlength])

    while True:
        suffix = random_str(8)
        filename_let = [temp_dir, '/.', os.path.basename(localfile), '.', suffix]
        temp_filename = ''.join(filename_let)
        if not os.path.exists(temp_filename):
            return temp_filename

if __name__ == '__main__':
    pass



#!/usr/bin/env python
#coding=utf8
import os
import time
import sys
import optparse
from optparse import OptionParser
from oss_api import *
from oss_xml_handler import *

DEBUG = False 
INTERVAL = 1
def check_res(res, msg, cost):
    request_id = res.getheader('x-oss-request-id')
    if (res.status / 100) == 2:
        if DEBUG:
            print "request_id:%s, %s OK, cost:%s ms" % (request_id, msg, cost)
            return 0
    else:
        print "request_id:%s, %s FAIL, cost:%s ms, ret:%s" % (request_id, msg, cost, res.status)
        return 1

def check(res, msg, begin_time):
    end_time = time.time()
    cost = "%.2f" % (1000*(end_time - begin_time))
    time.sleep(INTERVAL)
    return check_res(res, msg, cost)

if __name__ == "__main__": 
    parser = OptionParser()
    parser.add_option("-H", "--host", dest="host", help="specify ")
    parser.add_option("", "--id", dest="id", help="specify ")
    parser.add_option("", "--key", dest="key", help="specify ")
    parser.add_option("", "--bucket", dest="bucket", help="specify ")
    parser.add_option("", "--debug", dest="debug", help="specify ")
    parser.add_option("", "--interval", dest="interval", help="specify ")
    (opts, args) = parser.parse_args()
    if opts.debug is not None:
        DEBUG = True
    HOST = opts.host
    ID = opts.id
    KEY = opts.key
    BUCKET = opts.bucket
    if opts.interval:
        INTERVAL = (float)(opts.interval)
    if not HOST or not ID or not KEY or not BUCKET:
        print "python %s --host=xxx --id=xxx --key=xxx --bucket=xxx" % __file__
        print "For example: \npython %s --host=oss-cn-hangzhou.aliyuncs.com --id=your-id --key=your-key --bucket=testbucket%s" % (__file__, (int)(time.time()))
        exit(-1)
    for host in [HOST]:
        is_test_ok = False
        oss = OssAPI(host, ID, KEY)
        start_time = time.time()
        bucket = BUCKET 
        
        #bucket 相关接口
        #创建bucket
        b_ = time.time()
        acl = 'private'
        headers = {}
        res = oss.put_bucket(bucket, acl, headers)
        msg = "create bucket:%s" % (bucket)
        if check(res, msg, b_):
            break

        #获取bucket的权限
        b_ = time.time()
        res = oss.get_bucket_acl(bucket)
        msg = "get bucket acl:%s" % (bucket)
        if check(res, msg, b_):
            break

        #查看bucket所属的数据中心
        b_ = time.time()
        acl = 'private'
        headers = {}
        res = oss.get_bucket_location(bucket)
        msg = "get bucket location:%s" % (bucket)
        if check(res, msg, b_):
            break

        #设置bucket的CORS
        b_ = time.time()
        cors_xml = '''<CORSConfiguration>
                         <CORSRule>
                           <AllowedOrigin>http://www.example.com</AllowedOrigin>
                           <AllowedMethod>PUT</AllowedMethod>
                           <AllowedMethod>POST</AllowedMethod>
                           <AllowedMethod>DELETE</AllowedMethod>
                           <AllowedHeader>*</AllowedHeader>
                           <MaxAgeSeconds>3000</MaxAgeSeconds>
                           <ExposeHeader>test</ExposeHeader>
                           <ExposeHeader>test2</ExposeHeader>
                         </CORSRule>
                         <CORSRule>
                           <AllowedOrigin>*</AllowedOrigin>
                           <AllowedMethod>GET</AllowedMethod>
                         </CORSRule>
                      </CORSConfiguration>'''
        res = oss.put_cors(bucket, cors_xml)
        msg = "put cors:%s" % (bucket)
        if check(res, msg, b_):
            break

        #读取bucket的CORS
        b_ = time.time()
        res = oss.get_cors(bucket)
        msg = "get cors:%s" % (bucket)
        if check(res, msg, b_):
            break
        
        #删除bucket的CORS
        b_ = time.time()
        res = oss.delete_cors(bucket)
        msg = "delete cors:%s" % (bucket)
        if check(res, msg, b_):
            break

        #设置bucket的website
        b_ = time.time()
        index_file = "index.html"
        error_file = "404.html"
        res = oss.put_website(bucket, index_file, error_file)
        msg = "put website:%s" % (bucket)
        if check(res, msg, b_):
            break

        #读取bucket的website
        b_ = time.time()
        res = oss.get_website(bucket)
        msg = "get website:%s" % (bucket)
        if check(res, msg, b_):
            break

        #删除bucket的website
        b_ = time.time()
        res = oss.delete_website(bucket)
        msg = "delete website:%s" % (bucket)
        if check(res, msg, b_):
            break

        #设置bucket的lifecycle
        b_ = time.time()
        lifecycle = '''
            <LifecycleConfiguration>
              <Rule>
                <ID>1125</ID>
                <Prefix>12</Prefix>
                <Status>Enabled</Status>
                <Expiration>
                  <Days>2</Days>
                </Expiration>
              </Rule>
            </LifecycleConfiguration>'''
        res = oss.put_lifecycle(bucket, lifecycle)
        msg = "put lifecycle:%s" % (bucket)
        if check(res, msg, b_):
            break

        #读取bucket的lifecycle
        b_ = time.time()
        res = oss.get_lifecycle(bucket)
        msg = "get lifecycle:%s" % (bucket)
        if check(res, msg, b_):
            break
        
        #删除bucket的lifecycle
        b_ = time.time()
        res = oss.delete_lifecycle(bucket)
        msg = "delete lifecycle:%s" % (bucket)
        if check(res, msg, b_):
            break

        #设置bucket的logging
        b_ = time.time()
        prefix = "logging-prefix"
        res = oss.put_logging(bucket, bucket, prefix)
        msg = "put bucket logging:%s" % (bucket)
        if check(res, msg, b_):
            break
        
        #读取bucket的logging
        b_ = time.time()
        res = oss.get_logging(bucket)
        msg = "get bucket logging:%s" % (bucket)
        if check(res, msg, b_):
            break
        
        #删除bucket的logging
        b_ = time.time()
        res = oss.delete_logging(bucket)
        msg = "delete bucket logging:%s" % (bucket)
        if check(res, msg, b_):
            break

        #列出创建的bucket
        b_ = time.time()
        res = oss.get_service()
        msg = "get service"
        if check(res, msg, b_):
            break
        
        #object相关：
        #把指定的字符串内容上传到bucket中,在bucket中的文件名叫object。
        b_ = time.time()
        object = "object_test"
        input_content = "hello, OSS"
        content_type = "text/HTML"
        headers = {}
        res = oss.put_object_from_string(bucket, object, input_content, content_type, headers)
        msg = "put /%s/%s from string" % (bucket, object)
        if check(res, msg, b_):
            break 

        #调用copy接口
        b_ = time.time()
        headers = {}
        headers["x-oss-meta-test"] = "oss-test-meta"
        res = oss.copy_object(bucket, object, bucket, object, headers)
        msg = "copy /%s/%s " % (bucket, object)
        if check(res, msg, b_):
            break 
        
        #指定文件名, 把这个文件上传到bucket中,在bucket中的文件名叫object。
        b_ = time.time()
        filename = __file__ 
        content_type = "text/HTML"
        headers = {}
        res = oss.put_object_from_file(bucket, object, filename, content_type, headers)
        msg = "put /%s/%s from file" % (bucket, object)
        if check(res, msg, b_):
            break 
     
        #下载bucket中的object，内容在body中
        b_ = time.time()
        headers = {}
        res = oss.get_object(bucket, object, headers)
        msg = "get /%s/%s" % (bucket, object)
        if check(res, msg, b_):
            break

        #下载bucket中的object，把内容写入到本地文件中
        b_ = time.time()
        headers = {}
        filename = "get_object_test_file"
        res = oss.get_object_to_file(bucket, object, filename, headers)
        msg = "get /%s/%s to %s" % (bucket, object, filename)
        if filename:
            os.remove(filename)
        if check(res, msg, b_):
            break

        #查看object的meta 信息，例如长度，类型等
        b_ = time.time()
        headers = {}
        res = oss.head_object(bucket, object, headers)
        msg = "head /%s/%s" % (bucket, object)
        if check(res, msg, b_):
            break
        
        #列出bucket中所拥有的object
        b_ = time.time()
        prefix = ""
        marker = ""
        delimiter = "/"
        maxkeys = "100"
        headers = {}
        res = oss.get_bucket(bucket, prefix, marker, delimiter, maxkeys, headers)
        msg = "list bucket %s" % bucket
        if check(res, msg, b_):
            break
     
        #multipart相关
        #初始化一个upload_id
        b_ = time.time()
        res = oss.init_multi_upload(bucket, object)
        msg = "init multipart %s" % bucket
        if check(res, msg, b_):
            break
        body = res.read()
        h = GetInitUploadIdXml(body)
        upload_id = h.upload_id
        
        #删除upload_id相关的part
        b_ = time.time()
        res = oss.cancel_upload(bucket, object, upload_id)
        msg = "cancel multpart %s" % bucket
        if check(res, msg, b_):
            break

        #再次初始化
        b_ = time.time()
        res = oss.init_multi_upload(bucket, object)
        msg = "init multipart %s" % bucket
        if check(res, msg, b_):
            break
        body = res.read()
        h = GetInitUploadIdXml(body)
        upload_id = h.upload_id

        #上传part
        b_ = time.time()
        part_number = 1
        res = oss.upload_part(bucket, object, __file__, upload_id, part_number)
        msg = "upload part /%s/%s" % (bucket, object)
        if check(res, msg, b_):
            break
        
        #通过upload_id查看上传了多少块
        b_ = time.time()
        res = oss.get_all_parts(bucket, object, upload_id)
        msg = "get parts of /%s/%s by upload_id" % (bucket, object)
        if check(res, msg, b_):
            break

        #查看bucket中有多少正在上传的multipart
        b_ = time.time()
        res = oss.get_all_multipart_uploads(bucket, delimiter=None, max_uploads=None, key_marker=None, prefix=None, upload_id_marker=None)
        msg = "get all uploads in %s" % bucket
        if check(res, msg, b_):
            break

        #完成multipart上传
        b_ = time.time()
        part_msg_xml = get_part_xml(oss, bucket, object, upload_id)
        res = oss.complete_upload(bucket, object, upload_id, part_msg_xml)
        msg = "complete multipart /%s/%s" % (bucket, object)
        if check(res, msg, b_):
            break

        #删除bucket中的object
        b_ = time.time()
        headers = {}
        if (int)(b_) % 2 == 0:
            res = oss.delete_object(bucket, object, headers)
            msg = "delete object /%s/%s" % (bucket, object)
            if check(res, msg, b_):
                break
        else:
            res = oss.delete_objects(bucket, [object])
            msg = "delete objects in /%s" % (bucket)
            if check(res, msg, b_):
                break

        end_time = time.time()
        if DEBUG:
            print "check %s costs:%s s" % (host, (end_time - start_time)) 
        is_test_ok = True
    if is_test_ok:
        print "%s test OK" % host
    else:
        print "%s test FAIL" % host



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/6 10:04 下午
# @Author  : lance.txl
# @Site    : 
# @File    : db_utils.py
# @Software: PyCharm
import pymysql

class DBDao(object):
    def __init__(self):
        object.__init__(self)
    def set_params(self,host,user,password,database,charset='utf8'):
        self.host = host
        self.user = user
        self.password = password
        self.database = database
        self.charset = charset
    def execute_select_sql(self, sql):
        print(sql)
        db = self.init_db()
        cursor = db.cursor()
        try:
            cursor.execute(sql)
            l = cursor.fetchall()
        except Exception as e:
            print(e)
            l = []
            cursor.close()
            db.close()
        else:
            cursor.close()
            db.close()
        return l

    def execute_insert_sql(self, sql):
        print(sql)
        db = self.init_db()
        cursor = db.cursor()
        try:
            cursor.execute(sql)
            db.commit()
        except Exception as e:
            print("error")
            print(e)
            db.rollback()
            cursor.close()
            db.close()
        else:
            print("done")
            cursor.close()
            db.close()

    def init_db(self):
        db = pymysql.connect(host=self.host, user=self.user, password=self.password, database=self.database, charset=self.charset)
        return db


#!/usr/bin/env python
#coding=utf-8

# Copyright (C) 2011, Alibaba Cloud Computing

#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from xml.dom import minidom
import time
import sys
from xml.sax.saxutils import escape, unescape

XML_UNESCAPE_TABLE = {
    "&#26;" : ''
}

def get_md5():
    if sys.version_info >= (2, 6):
        import hashlib
        hash = hashlib.md5()
    else:
        import md5
        hash = md5.new()
    return hash

def get_xml_unescape_table():
    return XML_UNESCAPE_TABLE
    
def get_xml_unescape_map():
    xml_map = {}
    for k, v in XML_UNESCAPE_TABLE.items():
        m = get_md5()
        m.update(k)
        md5_k = m.hexdigest()
        xml_map[k] = md5_k + str(time.time())
    return xml_map

def has_tag(element, tag):
    nodes = element.getElementsByTagName(tag)
    if len(nodes):
        return True
    else:
        return False

def get_tag_text(element, tag, convert_to_bool = True):
    nodes = element.getElementsByTagName(tag)
    if len(nodes) == 0:
        return ""
    else:
        node = nodes[0]
    rc = ""
    for node in node.childNodes:
        if node.nodeType in ( node.TEXT_NODE, node.CDATA_SECTION_NODE):
            rc = rc + node.data
    if convert_to_bool:
        if rc == "true":
            return True
        elif rc == "false":
            return False
    return rc

class ErrorXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.code = get_tag_text(self.xml, 'Code')
        self.msg = get_tag_text(self.xml, 'Message')
        self.resource = get_tag_text(self.xml, 'Resource')
        self.request_id = get_tag_text(self.xml, 'RequestId')
        self.host_id = get_tag_text(self.xml, 'HostId')
    
    def show(self):
        print "Code: %s\nMessage: %s\nResource: %s\nRequestId: %s \nHostId: %s" % (self.code, self.msg, self.resource, self.request_id, self.host_id)

class Owner:
    def __init__(self, xml_element):
        self.element = xml_element
        self.id = get_tag_text(self.element, "ID")
        self.display_name = get_tag_text(self.element, "DisplayName")
    
    def show(self):
        print "ID: %s\nDisplayName: %s" % (self.id, self.display_name)

class Bucket:
    def __init__(self, xml_element):
        self.element = xml_element
        self.location = get_tag_text(self.element, "Location")
        self.name = get_tag_text(self.element, "Name", convert_to_bool = False)
        self.creation_date = get_tag_text(self.element, "CreationDate")
    
    def show(self):
        print "Name: %s\nCreationDate: %s\nLocation: %s" % (self.name, self.creation_date, self.location)

class GetServiceXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.owner = Owner(self.xml.getElementsByTagName('Owner')[0])
        self.buckets = self.xml.getElementsByTagName('Bucket')
        self.bucket_list = []
        self.prefix = get_tag_text(self.xml, 'Prefix', convert_to_bool = False)
        self.marker = get_tag_text(self.xml, 'Marker', convert_to_bool = False)
        self.maxkeys = get_tag_text(self.xml, 'MaxKeys')
        self.is_truncated = get_tag_text(self.xml, 'IsTruncated')
        self.nextmarker = get_tag_text(self.xml, 'NextMarker')

        for b in self.buckets:
            self.bucket_list.append(Bucket(b))

    def show(self):
        print "Owner:"
        self.owner.show()
        print "\nBucket list:"
        for b in self.bucket_list:
            b.show()
            print ""

    def list(self):
        bl = []
        for b in self.bucket_list:
            bl.append((b.name, b.creation_date, b.location))
        return bl
    
    def get_prefix(self):
        return self.prefix

    def get_marker(self):
        return self.marker

    def get_maxkeys(self):
        return self.maxkeys
    
    def get_istruncated(self):
        return self.is_truncated

    def get_nextmarker(self):
        return self.nextmarker
    
class Content:
    def __init__(self, xml_element):
        self.element = xml_element
        self.key = get_tag_text(self.element, "Key", convert_to_bool = False)        
        self.last_modified = get_tag_text(self.element, "LastModified")        
        self.etag = get_tag_text(self.element, "ETag")        
        self.size = get_tag_text(self.element, "Size")        
        self.owner = Owner(self.element.getElementsByTagName('Owner')[0])
        self.storage_class = get_tag_text(self.element, "StorageClass")        

    def show(self):
        print "Key: %s\nLastModified: %s\nETag: %s\nSize: %s\nStorageClass: %s" % (self.key, self.last_modified, self.etag, self.size, self.storage_class)
        self.owner.show()

class Part:
    def __init__(self, xml_element):
        self.element = xml_element
        self.part_num = get_tag_text(self.element, "PartNumber")        
        self.object_name = get_tag_text(self.element, "PartName")
        self.object_size = get_tag_text(self.element, "PartSize")
        self.etag = get_tag_text(self.element, "ETag")

    def show(self):
        print "PartNumber: %s\nPartName: %s\nPartSize: %s\nETag: %s\n" % (self.part_num, self.object_name, self.object_size, self.etag)

class PostObjectGroupXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.size = get_tag_text(self.xml, 'Size')
        self.etag = get_tag_text(self.xml, "ETag")

    def show(self):
        print "Post Object Group, Bucket: %s\nKey: %s\nSize: %s\nETag: %s" % (self.bucket, self.key, self.size, self.etag)

class GetObjectGroupIndexXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.etag = get_tag_text(self.xml, 'Etag')
        self.file_length = get_tag_text(self.xml, 'FileLength')
        self.index_list = []
        index_lists = self.xml.getElementsByTagName('Part')
        for i in index_lists:
            self.index_list.append(Part(i))

    def list(self):
        index_list = []
        for i in self.index_list:
            index_list.append((i.part_num, i.object_name, i.object_size, i.etag))
        return index_list

    def show(self):
        print "Bucket: %s\nObject: %s\nEtag: %s\nObjectSize: %s" % (self.bucket, self.key, self.etag, self.file_length)
        print "\nPart list:"
        for p in self.index_list:
            p.show()

class GetObjectLinkIndexXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.index_list = []
        index_lists = self.xml.getElementsByTagName('Part')
        for i in index_lists:
            self.index_list.append(Part(i))

    def list(self):
        index_list = []
        for i in self.index_list:
            index_list.append((i.part_num, i.object_name))
        return index_list

    def show(self):
        print "\nPart list:"
        for p in self.index_list:
            p.show()

class GetObjectLinkInfoXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.type = get_tag_text(self.xml, 'Type')
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.etag = get_tag_text(self.xml, 'ETag')
        self.last_modified = get_tag_text(self.xml, 'LastModified')
        self.index_list = []
        index_lists = self.xml.getElementsByTagName('Part')
        for i in index_lists:
            self.index_list.append(Part(i))

    def list(self):
        index_list = []
        for i in self.index_list:
            index_list.append((i.part_num, i.object_name, i.object_size, i.etag))
        return index_list

    def show(self):
        print "Bucket: %s\nType: %s\nObject: %s\nEtag: %s\nLastModified: %s" % (self.bucket, self.type, self.key, self.etag, self.last_modified)
        print "\nPart list:"
        for p in self.index_list:
            p.show()

class GetBucketXml:
    def __init__(self, xml_string):
        self.xml_unescape_table = {}
        self.xml_map = {}
        try:
            self.xml = minidom.parseString(xml_string)
        except:
            print xml_string
            self.xml_unescape_tabl = get_xml_unescape_table()
            self.xml_map = get_xml_unescape_map()
            for k, v in self.xml_map.items():
                xml_string = xml_string.replace(k, v)
            self.xml = minidom.parseString(xml_string)
        
        self.name = get_tag_text(self.xml, 'Name', convert_to_bool = False)
        self.prefix = get_tag_text(self.xml, 'Prefix', convert_to_bool = False)
        self.marker = get_tag_text(self.xml, 'Marker', convert_to_bool = False)
        self.nextmarker = get_tag_text(self.xml, 'NextMarker', convert_to_bool = False)
        self.maxkeys = get_tag_text(self.xml, 'MaxKeys')
        self.delimiter = get_tag_text(self.xml, 'Delimiter', convert_to_bool = False)
        self.is_truncated = get_tag_text(self.xml, 'IsTruncated')

        self.prefix_list = []
        prefixes = self.xml.getElementsByTagName('CommonPrefixes')
        for p in prefixes:
            tag_txt = get_tag_text(p, "Prefix")
            self.prefix_list.append(tag_txt)

        self.content_list = []
        contents = self.xml.getElementsByTagName('Contents')
        for c in contents:
            self.content_list.append(Content(c))

    def show(self):
        print "Name: %s\nPrefix: %s\nMarker: %s\nNextMarker: %s\nMaxKeys: %s\nDelimiter: %s\nIsTruncated: %s" % (self.name, self.prefix, self.marker, self.nextmarker, self.maxkeys, self.delimiter, self.is_truncated)
        print "\nPrefix list:"
        for p in self.prefix_list:
            print p
        print "\nContent list:"
        for c in self.content_list:
            c.show()
            print ""

    def list(self):
        cl = []
        pl = []
        for c in self.content_list:
            key = c.key
            if self.xml_map:
                for k, v in self.xml_map.items():
                    key = key.replace(v, k)
                key = unescape(key, self.xml_unescape_table)
            cl.append((key, c.last_modified, c.etag, c.size, c.owner.id, c.owner.display_name, c.storage_class))
        for p in self.prefix_list:
            pl.append(p)

        return (cl, pl)
 
class GetBucketAclXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        if len(self.xml.getElementsByTagName('Owner')) != 0:
            self.owner = Owner(self.xml.getElementsByTagName('Owner')[0])
        else:
            self.owner = "" 
        self.grant = get_tag_text(self.xml, 'Grant')

    def show(self):
        print "Owner Name: %s\nOwner ID: %s\nGrant: %s" % (self.owner.id, self.owner.display_name, self.grant)
 
class GetBucketLocationXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.location = get_tag_text(self.xml, 'LocationConstraint')
    
    def show(self):
        print "LocationConstraint: %s" % (self.location)

class GetInitUploadIdXml:
    def __init__(self, xml_string):
        self.xml_unescape_table = {}
        self.xml_map = {}
        try:
            self.xml = minidom.parseString(xml_string)
        except:
            print xml_string
            self.xml_unescape_tabl = get_xml_unescape_table()
            self.xml_map = get_xml_unescape_map()
            for k, v in self.xml_map.items():
                xml_string = xml_string.replace(k, v)
            self.xml = minidom.parseString(xml_string)
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.object = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        if self.xml_map:
            for k, v in self.xml_map.items():
                self.object = self.object.replace(v, k)
            self.object = unescape(self.object, self.xml_unescape_table)
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.upload_id = get_tag_text(self.xml, 'UploadId')
        self.marker = get_tag_text(self.xml, 'Marker', convert_to_bool = False)
       
    def show(self):
        print " "     

class Upload:
    def __init__(self, xml_element):
        self.element = xml_element
        self.key = get_tag_text(self.element, "Key", convert_to_bool = False)        
        self.upload_id = get_tag_text(self.element, "UploadId")
        self.init_time = get_tag_text(self.element, "Initiated")

class GetMultipartUploadsXml:
    def __init__(self, xml_string):
        self.xml_unescape_table = {} 
        self.xml_map = {}
        try:
            self.xml = minidom.parseString(xml_string)
        except:
            self.xml_unescape_tabl = get_xml_unescape_table()
            self.xml_map = get_xml_unescape_map()
            for k, v in self.xml_map.items():
                xml_string = xml_string.replace(k, v)
            self.xml = minidom.parseString(xml_string)
        
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.key_marker = get_tag_text(self.xml, 'KeyMarker', convert_to_bool = False)
        self.upload_id_marker = get_tag_text(self.xml, 'UploadIdMarker')
        self.next_key_marker = get_tag_text(self.xml, 'NextKeyMarker', convert_to_bool = False)
        self.next_upload_id_marker = get_tag_text(self.xml, 'NextUploadIdMarker')
        self.delimiter = get_tag_text(self.xml, 'Delimiter', convert_to_bool = False)
        self.prefix = get_tag_text(self.xml, 'Prefix', convert_to_bool = False)
        self.max_uploads = get_tag_text(self.xml, 'MaxUploads')
        self.is_truncated = get_tag_text(self.xml, 'IsTruncated')

        self.prefix_list = []
        prefixes = self.xml.getElementsByTagName('CommonPrefixes')
        for p in prefixes:
            tag_txt = get_tag_text(p, "Prefix")
            self.prefix_list.append(tag_txt)

        self.content_list = []
        contents = self.xml.getElementsByTagName('Upload')
        for c in contents:
            self.content_list.append(Upload(c))

    def list(self):
        cl = []
        pl = []
        for c in self.content_list:
            key = c.key
            if self.xml_map:
                for k, v in self.xml_map.items():
                    key = key.replace(v, k)
                key = unescape(key, self.xml_unescape_table)
            cl.append((key, c.upload_id, c.init_time))
        for p in self.prefix_list:
            pl.append(p)

        return (cl, pl)

class MultiPart:
    def __init__(self, xml_element):
        self.element = xml_element
        self.part_number = get_tag_text(self.element, 'PartNumber')
        self.last_modified = get_tag_text(self.element, 'LastModified')
        self.etag = get_tag_text(self.element, 'ETag')
        self.size = get_tag_text(self.element, 'Size')

class GetPartsXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.upload_id = get_tag_text(self.xml, 'UploadId')
        self.storage_class = get_tag_text(self.xml, 'StorageClass')
        self.next_part_number_marker = get_tag_text(self.xml, 'NextPartNumberMarker')
        self.max_parts = get_tag_text(self.xml, 'MaxParts')
        self.is_truncated = get_tag_text(self.xml, 'IsTruncated')
        self.part_number_marker = get_tag_text(self.xml, 'PartNumberMarker')
        
        self.content_list = []
        contents = self.xml.getElementsByTagName('Part')
        for c in contents:
            self.content_list.append(MultiPart(c))

    def list(self):
        cl = []
        for c in self.content_list:
            cl.append((c.part_number, c.etag, c.size, c.last_modified))
        return cl

class CompleteUploadXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.location = get_tag_text(self.xml, 'Location')
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.etag = get_tag_text(self.xml, "ETag")

class DeletedObjectsXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        contents = self.xml.getElementsByTagName('Deleted')
        self.content_list = []
        for c in contents:
            self.content_list.append(get_tag_text(c, 'Key', convert_to_bool = False))
    def list(self):
        cl = []
        for c in self.content_list:
            cl.append(c)
        return cl

class CnameInfoPart:
    def __init__(self, xml_element):
        self.element = xml_element
        self.cname = get_tag_text(self.element, 'Cname')
        self.bucket = get_tag_text(self.element, 'Bucket', convert_to_bool = False)
        self.status = get_tag_text(self.element, 'Status')
        self.lastmodifytime = get_tag_text(self.element, 'LastModifyTime')

class CnameToBucketXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.content_list = []
        contents = self.xml.getElementsByTagName('CnameInfo')
        for c in contents:
            self.content_list.append(CnameInfoPart(c))

    def list(self):
        cl = []
        for c in self.content_list:
            cl.append((c.cname, c.bucket, c.status, c.lastmodifytime))
        return cl

class RedirectXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.endpoint = get_tag_text(self.xml, 'Endpoint')
    def Endpoint(self):
        return self.endpoint

class PostObjectResponseXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.object= get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.etag = get_tag_text(self.xml, 'ETag')
        self.location = get_tag_text(self.xml, 'Location')

    def show(self):
        print "Bucket: %s\nObject: %s\nEtag: %s\nLocation: %s" % (self.bucket, self.key, self.etag, self.location)

class WebsiteXml:
    def __init__(self, xml_element):
        self.element = minidom.parseString(xml_element) 
        self.index_file = get_tag_text(self.element, 'Suffix', convert_to_bool = False)
        self.error_file = get_tag_text(self.element, 'Key', convert_to_bool = False)

class Rule:
    def __init__(self, xml_element):
        self.max_age = ""
        self.max_age = get_tag_text(xml_element, 'MaxAgeSeconds')
        def get_list_by_tag(xml_element, tag):
            list = []
            nodes = xml_element.getElementsByTagName(tag)
            for node in nodes:
                for tmp_node in node.childNodes:
                    if tmp_node.nodeType in (tmp_node.TEXT_NODE, tmp_node.CDATA_SECTION_NODE):
                        list.append(tmp_node.data)
            return list
        self.method_list = get_list_by_tag(xml_element, 'AllowedMethod')
        self.origin_list = get_list_by_tag(xml_element, 'AllowedOrigin') 
        self.header_list = get_list_by_tag(xml_element, 'AllowedHeader')
        self.expose_header_list = get_list_by_tag(xml_element, 'ExposeHeader') 

    def show(self):
        print "max_age:%s" % self.max_age
        print "method_list:"
        for i in self.method_list:
            print "%s" % i
        print "origin_list:"
        for i in self.origin_list:
            print "%s" % i
        print "header_list:"
        for i in self.header_list:
            print "%s" % i
        print "expose_header_list:"
        for i in self.expose_header_list:
            print "%s" % i
    def get_msg(self):
        msg = "max_age:%s" % self.max_age
        msg += "method_list:"
        for i in sorted(self.method_list):
            msg += "%s" % i
        msg += "origin_list:"
        for i in sorted(self.origin_list):
            msg += "%s" % i
        msg += "header_list:"
        for i in sorted(self.header_list):
            msg += "%s" % i
        msg += "expose_header_list:"
        for i in sorted(self.expose_header_list):
            msg += "%s" % i
        return msg

class CorsXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        rules = self.xml.getElementsByTagName('CORSRule')
        self.rule_list = []
        for rule in rules:
            self.rule_list.append(Rule(rule))

class RefererXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.allow_empty_referer = get_tag_text(self.xml, "AllowEmptyReferer")
        self.referer_list = []
        referers = self.xml.getElementsByTagName('RefererList')[0]
        names = referers.getElementsByTagName('Referer')
        for name in names:
            for child in name.childNodes:
                r = child.nodeValue
                self.referer_list.append(r)

class LoggingXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.target_bucket = get_tag_text(self.xml, "TargetBucket")
        self.prefix = get_tag_text(self.xml, "TargetPrefix")

class LifecycleRule:
    def __init__(self, xml_element):
        self.element = xml_element
        self.id = get_tag_text(self.element, "ID")
        self.prefix = get_tag_text(self.element, "Prefix")
        self.status = get_tag_text(self.element, "Status")
        self.date = get_tag_text(self.element, "Date", '')
        self.days = get_tag_text(self.element, "Days", '')
    
    def show(self):
        print "ID: %s\nPrefix: %s\nStatus: %s\nExpiration: %s\n" % (self.id, self.prefix, self.status, self.expiration)

class LifecycleXml:
    def __init__(self, xml_string):
        self.xml = minidom.parseString(xml_string)
        self.rules = self.xml.getElementsByTagName('Rule')
        self.rule_list = []
        for r in self.rules:
            self.rule_list.append(LifecycleRule(r))

    def show(self):
        print "\nRule list:"
        for b in self.rule_list:
            b.show()

class GetObjectInfoXml:
    def __init__(self, xml_string):
        try:
            self.xml = minidom.parseString(xml_string)
        except:
            print xml_string
        self.bucket = get_tag_text(self.xml, 'Bucket', convert_to_bool = False)
        self.type = get_tag_text(self.xml, 'Type', convert_to_bool = False)
        self.key = get_tag_text(self.xml, 'Key', convert_to_bool = False)
        self.last_modified = get_tag_text(self.xml, 'LastModified', convert_to_bool = False)
        self.etag = get_tag_text(self.xml, 'ETag', convert_to_bool = False)
        self.content_type = get_tag_text(self.xml, 'Content-Type')
        self.size = get_tag_text(self.xml, 'Size', convert_to_bool = False)
        self.parts = []
        parts = self.xml.getElementsByTagName('Part')
        for p in parts:
            self.parts.append(Part(p))

    def show(self):
        print "Bucket: %s\nType: %s\nKey: %s\nLastModified: %s\nETag: %s\nContent-Type: %s\nSize: %s" % (self.bucket, self.type, self.key, self.last_modified, self.etag, self.content_type, self.size)

if __name__ == "__main__":
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/6 10:02 下午
# @Author  : lance.txl
# @Site    : 
# @File    : adb_utils.py
# @Software: PyCharm

import os

def get_serial_list():
    cmd = "adb devices"
    lines = os.popen(cmd).readlines()
    devices = [x.split("\t")[0].strip() for x in lines[1:] if re.search("device",x)]
    return devices

def adb_head(serial):
    if serial:
        return "adb -s {}".format(serial)
    else:
        return "adb"

def get_new_current_activity(topseq=0,serial=None):
    cmd = "{} shell dumpsys activity | grep -i 'Run #.: ActivityRecord'".format(adb_head(serial))
    lines = [x.strip() for x in os.popen(cmd).readlines() if x.strip()]
    if lines:
        parts = [x.strip() for x in lines[topseq].split(" ") if x.strip()]
        return parts[4] if len(parts) > 5 else ""
    return ""


if __name__ == '__main__':
    r = get_new_current_activity()
    print(r)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/14 10:25 上午
# @Author  : lance.txl
# @Site    : 
# @File    : date_utils.py
# @Software: PyCharm

from datetime import datetime,timedelta

def get_date_tag():
    """
    获取一个时间tag
    :return:
    """
    now = datetime.now()
    s = now.strftime("%Y%m%d-%H%M")
    return s
if __name__ == '__main__':
    get_date_tag()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/12 8:06 下午
# @Author  : lance.txl
# @Site    : 
# @File    : oss_utils.py
# @Software: PyCharm

import re
import time
import socket
import requests

import oss2
import os
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
from pprint import pprint
from urllib import request

OSS_DIR = "public/OSS_Python_API_20150707"
if not os.path.isdir(os.path.abspath(OSS_DIR)):
    OSS_DIR = os.path.dirname(os.path.abspath(__file__)) + "/" + OSS_DIR
print(OSS_DIR)
class OssUtils:
    def __init__(self,bucketname):
        self.bucketname = bucketname
        if not os.path.isdir(OSS_DIR):
            print("Not Found {} Directly,Please Assign later by use set_ossenv".format(OSS_DIR))
        else:
            self.upload_cmd = "python2 " + OSS_DIR + "/osscmd upload {} oss://" + bucketname + "/{}"
            print(self.upload_cmd)
    def set_ossenv(self,ossdir):
        self.upload_cmd = "python2 " + ossdir + "/osscmd upload {} oss://" + self.bucketname + "/{}"
        print(self.upload_cmd)
    def upload_oss_file(self,file,filename=None,delete_file=False):
        if not filename:
            filename = file.split("/")[-1]
        cmd = self.upload_cmd.format(file,filename)
        print(cmd)
        lines = os.popen(cmd).readlines()
        pprint(lines)
        download_url = ""
        for index in range(len(lines)):
            if lines[index].startswith("Object URL"):
                download_url = lines[index].strip().split("is:")[-1].strip()
        if download_url and delete_file:
            print("upload success,remove screenshot")
            os.remove(file)
        return download_url
    def download_file(self,url,filename=None):
        if not filename:
            filename = url.split("/")[-1]
        request.urlretrieve(url,filename)
    def download_file_multithread(self,p):
        url,filename = p.split("~")
        if not filename:
            filename = url.split("/")[-1]
        request.urlretrieve(url,filename)
    def download_file_withtimeout(self,url,filename=None,timeout=10):
        socket.setdefaulttimeout(timeout)
        if not filename:
            filename = url.split("/")[-1]
        count = 0
        while count < 3:
            try:
                r = requests.get(url,timeout=timeout)
            except requests.exceptions.RequestException as e:
                print(e)
                count = count + 1
                continue
            else:
                with open(filename, "wb") as fo:
                    fo.write(r.content)
                break



if __name__ == '__main__':
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/12 7:51 下午
# @Author  : lance.txl
# @Site    : 
# @File    : odps_utils.py
# @Software: PyCharm

from datetime import datetime,timedelta
from mq_utils.db_utils import DBDao
import requests
from pprint import pprint
class GetODPSData:
    def __init__(self,url):
        # 拼接时间和偏移量
        self.url = url
        self.db = DBDao()
    def set_db_params(self,host,user,password,database):
        self.db.set_params(host=host,user=user,password=password,database=database)
    def get_today(self):
        today = datetime.now().strftime("%Y%m%d")
        return today
    def get_yesterday(self):
        today = datetime.now()
        yesterday = today - timedelta(days=1)
        return yesterday.strftime("%Y%m%d")
    def get_before_yesterday(self):
        today = datetime.now()
        before_yesterday = today - timedelta(days=2)
        return before_yesterday.strftime("%Y%m%d")
    def get_response(self,url):
        r = requests.get(url)
        data = r.json()
        if not data or 'data' not in data:
            return None
        return data.get("data")
    def get_newest_queryodps(self,dt=""):
        if not dt:
            sql = "select dt,omit from get_odps_record order by id desc limit 1"
        else:
            sql = "select dt,omit from get_odps_record where dt=\"{}\" order by id desc limit 1".format(dt)
        l = self.db.execute_select_sql(sql)
        print(l)
        if l:
            return l[0]
        return l
    def insert_newest_queryodps(self,dt,omit):
        sql = "insert into get_odps_record(dt,omit) values (\"{}\",\"{}\")".format(dt,omit)
        self.db.execute_insert_sql(sql)

    def get(self):
        """
        调用获取2000条场所码数据
        @:param newest_data : (dt,omit)
        :return:
        """
        # 从数据库获取当前已经取的次数
        newest_data = self.get_newest_queryodps()
        dt = self.get_yesterday()
        omit = 0
        if newest_data:
            db_dt,db_omit = newest_data
            if db_dt == self.get_yesterday():
                omit = int(db_omit) + 1
        # 拼接url，获取到结果，归一化
        url = self.url.format(dt,omit)
        print(url)
        datas = self.get_response(url)
        pprint(datas)
        # 默认是获取昨天的数据，可能由于数据清洗过晚的缘故问题
        if not datas:
            # 补充获取前天的数据
            dt = self.get_before_yesterday()
            try_r = self.get_newest_queryodps(dt=dt)
            if not try_r:
                omit = 0
            else:
                omit = try_r[1]
            omit = int(omit) + 1
            url = self.url.format(dt, omit)
            print(url)
            datas = self.get_response(url)
        if not datas:
            # 取两天还是没有，直接返回
            return None
        self.insert_newest_queryodps(dt, omit)
        return datas
if __name__ == '__main__':
    pass




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/6 9:06 下午
# @Author  : lance.txl
# @Site    : 
# @File    : utils.py
# @Software: PyCharm

import os
import hashlib
import cv2
import random
import re

def ergodicDir(Dir, FileType=""):
    fileDict = {}
    for parent, dirnames, filenames in os.walk(Dir):
        for i in filenames:
            if FileType != "":
                if i.split(".", -1)[-1] == FileType:
                    fileDict[os.path.join(parent, i)] = os.path.join(parent, i)
            else:
                fileDict[os.path.join(parent, i)] = os.path.join(parent, i)
    return fileDict

def get_file_content(filename):
    """
    仅支持最简单的按行读取逻辑
    :param filename:
    :return:
    """
    with open(filename,"r") as fi:
        lines = fi.readlines()
    return [x.strip() for x in lines if x.strip()]
def get_dir_files(pd,is_file=False):
    r = []
    if is_file:
        r = [os.path.join(pd,x) for x in os.listdir(pd) if not re.search("DS_Store",x) and os.path.isfile(os.path.join(pd,x))]
    else:
        r = [os.path.join(pd,x) for x in os.listdir(pd) if not re.search("DS_Store",x) and os.path.isdir(os.path.join(pd,x))]
    return r


def get_md5(s):
    m = hashlib.md5()
    m.update(s.encode("utf-8"))
    unique_id = m.hexdigest()
    return unique_id

def show_or_save_pic(pic,savename=None):
    if isinstance(pic,str):
        pic = cv2.imread(pic)
    name = "windowname_" + random.choice(list("abcdefg"))
    print(name)
    cv2.namedWindow(name,cv2.WINDOW_NORMAL)
    cv2.imshow(name,pic)
    k = cv2.waitKey(0)
    if k == 27:
        cv2.destroyWindow(name)
    elif k == ord("s"):
        if savename:
            cv2.imwrite(savename,pic)
        else:
            cv2.imwrite(name + ".png",pic)
        cv2.destroyWindow(name)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/12 8:08 下午
# @Author  : lance.txl
# @Site    : 
# @File    : mqpool_utils.py
# @Software: PyCharm

import threadpool
from multiprocessing import cpu_count,Pool
import random
import time

class MqProcessPool():
    def __init__(self, processnum=0):
        self.processnum = cpu_count() if not processnum else processnum
        self.func = None
        self.data = self.__example_data()
        self.callback = self.__print_result
        self.pp = Pool(processes=self.processnum)
        # self.pp = Pool()

    def set_callback(self, func):
        self.callback = func

    def set_run_func(self, func):
        self.func = func

    def set_missiondatas(self, datas):
        self.data = datas

    def __example_data(self):
        return [random.randint(1,10) for i in range(10)]

    def __print_result(self, request):
        print(request)
        # print("**** Result from request #%s: %r" % (request.requestID, result))

    def __handle_exception(self, request):
        print(request)
    def __getstate__(self):
        self_dict = self.__dict__.copy()
        print(self_dict)
        del self_dict['pp']
        return self_dict
    def __setstate__(self, state):
        self.__dict__.update(state)

    def run(self):
        # for d in self.data:
        #     self.pp.apply(self.func,args=(d,))
        self.pp.map_async(self.func,self.data,callback=self.callback,error_callback=self.__handle_exception)
        self.pp.close()
        self.pp.join()
        print("done")
class MqThreadPool:
    def __init__(self,threadnum=0):
        self.threadnum = cpu_count() if not threadnum else threadnum
        self.single_param = False
        self.func = self.__do_something
        self.data = self.__example_data()
        self.callback = self.__print_result
        self.tp = threadpool.ThreadPool(num_workers=self.threadnum)
    def set_callback(self,func):
        self.callback = func
    def set_run_func(self,func):
        self.func = func
    def set_missiondatas(self,datas):
        self.data = datas
    def __example_data(self):
        # return [random.randint(1,10) for i in range(20)]
        return [((random.randint(1,10),), {}) for i in range(20)]
    def __do_something(self,data):
        time.sleep(random.randint(1,5))
        result = round(random.random()*data, 5)
        if result > 5:
            raise RuntimeError("Something extraordinary happened!")
        return result

    def __print_result(self,request, result):
        pass
        # print("**** Result from request #%s: %r" % (request.requestID, result))

    def __handle_exception(self,request, exc_info):
        if not isinstance(exc_info, tuple):
            # Something is seriously wrong...
            print(request)
            print(exc_info)
            raise SystemExit
        print("**** Exception occured in request #%s: %s" % \
          (request.requestID, exc_info))
    def run(self):

        requests = threadpool.makeRequests(self.func, self.data, self.callback, self.__handle_exception)
        for req in requests:
            self.tp.putRequest(req)
            # print("Work request #%s added." % req.requestID)
        i = 0
        while True:
            try:
                time.sleep(0.5)
                self.tp.poll()
            except KeyboardInterrupt:
                print("**** Interrupted!")
                break
            except Exception:
                break
        if self.tp.dismissedWorkers:
            self.tp.joinAllDismissedWorkers()
